import * as React from 'react';
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { IgrToolCommandEventArgs } from "./igr-tool-command-event-args";
import { IgrToolActionPerformedEventArgs } from "./igr-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-react-core";
import { NamePatcher, getModifiedProps, isValidProp, toSpinal, initializePropertiesFromCss } from "igniteui-react-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export class IgrToolAction extends React.Component {
    constructor(props) {
        super(props);
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        this._performed = null;
        this._performed_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this.onImplementationCreated();
    }
    createImplementation() {
        return new ToolAction();
    }
    /**
     * @hidden
     */
    get i() {
        return this._implementation;
    } /**
                                 * @hidden
                                 */
    static _createFromInternal(internal) {
        if (!internal) {
            return null;
        }
        if (!internal.$type) {
            return null;
        }
        let name = internal.$type.name;
        let externalName = "Igr" + name;
        if (!TypeRegistrar.isRegistered(externalName)) {
            return null;
        }
        return TypeRegistrar.create(externalName);
    }
    onImplementationCreated() {
    }
    componentDidMount() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    render() {
        return null;
    }
    get id() {
        return this.i.ao;
    }
    get parentId() {
        return this.i.ax;
    }
    set parentId(v) {
        this.i.ax = v;
    }
    get commandId() {
        return this.i.ai;
    }
    set commandId(v) {
        this.i.ai = v;
    }
    get commandArgument() {
        return this.i.af;
    }
    set commandArgument(v) {
        this.i.af = v;
    }
    get commandArgumentValue() {
        return this.i.v;
    }
    set commandArgumentValue(v) {
        this.i.v = v;
    }
    get paddingLeft() {
        return this.i.r;
    }
    set paddingLeft(v) {
        this.i.r = +v;
    }
    get paddingRight() {
        return this.i.s;
    }
    set paddingRight(v) {
        this.i.s = +v;
    }
    get paddingBottom() {
        return this.i.q;
    }
    set paddingBottom(v) {
        this.i.q = +v;
    }
    get paddingTop() {
        return this.i.t;
    }
    set paddingTop(v) {
        this.i.t = +v;
    }
    get name() {
        return this.i.ar;
    }
    set name(v) {
        this.i.ar = v;
    }
    get label() {
        return this.i.ap;
    }
    set label(v) {
        this.i.ap = v;
    }
    get iconName() {
        return this.i.am;
    }
    set iconName(v) {
        this.i.am = v;
    }
    get iconCollectionName() {
        return this.i.ak;
    }
    set iconCollectionName(v) {
        this.i.ak = v;
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolAction");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    fromInfo(info) {
        this.i.a6(info);
    }
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
    get performed() {
        return this._performed;
    }
    set performed(ev) {
        if (this._performed_wrapped !== null) {
            this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
            this._performed_wrapped = null;
            this._performed = null;
        }
        this._performed = ev;
        this._performed_wrapped = (o, e) => {
            let outerArgs = new IgrToolActionPerformedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforePerformed) {
                this.beforePerformed(this, outerArgs);
            }
            if (this._performed) {
                this._performed(this, outerArgs);
            }
        };
        this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
        ;
    }
}
